"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.Type = void 0;
const typescript_1 = require("typescript");
var Type;
(function (Type) {
    Type["OBJECT"] = "object";
    Type["VARIABLE"] = "variable";
})(Type || (exports.Type = Type = {}));
const factory = () => {
    const extractElement = (element) => {
        if ((0, typescript_1.isIdentifier)(element.name)) {
            const result = (0, typescript_1.isBindingElement)(element) && element.propertyName
                ? element.propertyName.getText()
                : element.name.text;
            return {
                type: Type.VARIABLE,
                result,
            };
        }
        if ((0, typescript_1.isObjectBindingPattern)(element.name)) {
            const result = element.name.elements.map(extractElement).filter(Boolean);
            return {
                type: Type.OBJECT,
                result,
            };
        }
    };
    const extract = (fn) => {
        const source = stringify(fn);
        const sourceFile = (0, typescript_1.createSourceFile)('extractor.ts', source, typescript_1.ScriptTarget.ES5, true);
        const visit = (node) => {
            if ((0, typescript_1.isFunctionLike)(node)) {
                return node.parameters.flatMap(extractElement);
            }
            return (0, typescript_1.forEachChild)(node, visit);
        };
        return visit(sourceFile);
    };
    const stringify = (fn) => {
        return Function.prototype.toString.call(fn);
    };
    return {
        extract,
    };
};
exports.default = factory;
